from django.shortcuts import render
from brfinanceApp.models import *
from brfinanceApp.serializers import *
from django.contrib.auth.decorators import login_required,user_passes_test
from django.utils.decorators import method_decorator
from django.shortcuts import render,redirect
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from .serializers import *
import json
from clmainApp.utils import getBranchId


class ExpenseItemsApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_expenseitem'), login_url='/auth/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        all_expense_items =ExpenseItem.objects.filter(branch_id=branchId)
        all_expense_items_serialized=ExpenseItemSerializer(all_expense_items,many=True)
        return render(request,"Branch/brfinanceApp/expense_items.html",{"all_expense_items":all_expense_items_serialized.data,
        "all_expense_cats":ExpenseCategory.objects.all(),
        'activePage':'ExpenseItems'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_expenseitem'), login_url='/auth/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return Response(status=status.HTTP_400_BAD_REQUEST)
        request.data._mutable = True
        request.data.update({"branch":branchId})
        serializedObj= ExpenseItemSerializer(data=request.data)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

class ExpenseItemApi(APIView):
    def get_object(self,pk):
        try:
            return ExpenseItem.objects.get(pk=pk)
        except ExpenseItem.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_expenseitem'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        expense_itemObj=self.get_object(pk)
        if expense_itemObj :
            serializeobj= ExpenseItemSerializer(expense_itemObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_expenseitem'), login_url='/auth/login/'))
    def put(self,request,pk):
        branchId = getBranchId(request)
        obj=self.get_object(pk)
        request.data._mutable = True
        request.data.update({"created_by":request.user.id,"branch":branchId})
        serializedObj= ExpenseItemSerializer(obj,data=request.data)
        #list_error=""
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

        
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_expenseitem'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

############## MedicalService%Module #################

class MedicalServicesApi(APIView):
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_medicalservice'), login_url='/auth/login/'))
    def get(self,request):
        branchId = getBranchId(request)
        if not branchId : return redirect('clauthApp:home')
        all_medical_services =MedicalService.objects.filter(branch_id=branchId)
        #serializer=MedicalServiceSerializer(all_medical_services,many=True)
        return render(request,"Branch/brfinanceApp/medical_services.html",{
            "all_medical_services":all_medical_services,
            'activePage':'MedicalServices'})
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.add_medicalservice'), login_url='/auth/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return Response(status=status.HTTP_400_BAD_REQUEST)
        request.data._mutable = True
        request.data.update({"created_by":request.user.id,"branch":branchId})
        serializedObj= MedicalServiceSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(serializedObj.data["id"],status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)
class MedicalServiceApi(APIView):
    def get_object(self,pk):
        try:
            return MedicalService.objects.get(pk=pk)
        except MedicalService.DoesNotExist:
            return None    
    
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.view_medicalservice'), login_url='/auth/login/'))        
    def get(self,request,pk): 
        medical_serviceObj=self.get_object(pk)
        if medical_serviceObj :
            serializeobj= MedicalServiceSerializer(medical_serviceObj)
            return Response(serializeobj.data,status=status.HTTP_200_OK)
        else : return Response(status=status.HTTP_404_NOT_FOUND)

    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.change_medicalservice'), login_url='/auth/login/'))
    def put(self,request,pk):
        obj=self.get_object(pk)
        request.data._mutable = True
        request.data.update({"created_by":request.user.id})
        serializedObj= MedicalServiceSerializer(obj,data=request.data,partial=True)
        #serializedObj= MedicalServiceSerializer(obj,data=request.data,partial=True)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_200_OK)
        else : return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)

            
    @method_decorator(user_passes_test(lambda u: u.has_perm('BrManagement.delete_medicalservice'), login_url='/auth/login/'))
    def delete(self,request,pk):
        obj=self.get_object(pk)
        if obj :
            try:
                obj.delete()
                return Response(status=status.HTTP_204_NO_CONTENT)
            except Exception as err_msg: 
                    return Response(data ="{}".format(str(err_msg)),status=status.HTTP_405_METHOD_NOT_ALLOWED)
        else : return Response(status=status.HTTP_404_NOT_FOUND)
############## END MedicalService%Module #################

class ExpensesApi(APIView):
    # @method_decorator(user_passes_test(lambda u: u.has_perm('screen.add_patients'), login_url='/main/login/'))    
    def post(self,request):
        branchId = getBranchId(request)
        if not branchId : return Response(status=status.HTTP_400_BAD_REQUEST)
        request.data._mutable = True
        request.data.update({"created_by":request.user.id,'branch':branchId})
        serializedObj= ExpenseSerializer(data=request.data)
        if serializedObj.is_valid():
            serializedObj.save()
            return Response(status=status.HTTP_201_CREATED)
        else :return Response(serializedObj.errors,status=status.HTTP_400_BAD_REQUEST)